#ifndef _COMMANDS_H
#define _COMMANDS_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "callbacks.h"

// ----------------------
// Helper stuff

#define MAX_TYPE 6
#define MAX_SUBTYPE 100

enum Message{
	BEGIN_CHAR = 0xBE,
	END_CHAR   = 0xED
};

// This should also have double to avoid confusion with float values.
enum DataType
{
	floatType,
	intType,
	stringType
};

enum MessageTypeID{
	DEBUG_TYPE_ID             = 0x00,
	SETTER_TYPE_ID            = 0x01,
	GETTER_TYPE_ID            = 0x02,
	RESP_TYPE_ID              = 0x03,
	UPDATE_TYPE_ID            = 0x04,
	LOG_TYPE_ID               = 0x05
};

enum MessageSubTypeID{
	// DEBUG
	DEBUG_SUBTYPE_ID          = 0x00,
	PACKETLOG_SUBTYPE_ID      = 0x01,
	GETPACKETLOGS_SUBTYPE_ID  = 0x02,
	// SETTER
	// angle
	SETROLLP_SUBTYPE_ID       = 0x00,
	SETROLLI_SUBTYPE_ID       = 0x01,
	SETROLLD_SUBTYPE_ID       = 0x02,
	SETPITCHP_SUBTYPE_ID      = 0x03,
	SETPITCHI_SUBTYPE_ID      = 0x04,
	SETPITCHD_SUBTYPE_ID      = 0x05,
	SETYAWP_SUBTYPE_ID        = 0x06,
	SETYAWI_SUBTYPE_ID        = 0x07,
	SETYAWD_SUBTYPE_ID        = 0x08,
	// rate
	SETROLLRATEP_SUBTYPE_ID   = 0x09,
	SETROLLRATEI_SUBTYPE_ID   = 0x0A,
	SETROLLRATED_SUBTYPE_ID   = 0x0B,
	SETPITCHRATEP_SUBTYPE_ID  = 0x0C,
	SETPITCHRATEI_SUBTYPE_ID  = 0x0D,
	SETPITCHRATED_SUBTYPE_ID  = 0x0E,
	SETYAWRATEP_SUBTYPE_ID    = 0x0F,
	SETYAWRATEI_SUBTYPE_ID    = 0x10,
	SETYAWRATED_SUBTYPE_ID    = 0x11,
	// local pos
	SETLOCALXP_SUBTYPE_ID     = 0x12,
	SETLOCALXI_SUBTYPE_ID     = 0x13,
	SETLOCALXD_SUBTYPE_ID     = 0x14,
	SETLOCALYP_SUBTYPE_ID     = 0x15,
	SETLOCALYI_SUBTYPE_ID     = 0x16,
	SETLOCALYD_SUBTYPE_ID     = 0x17,
	SETALTP_SUBTYPE_ID        = 0x18,
	SETALTI_SUBTYPE_ID        = 0x19,
	SETALTD_SUBTYPE_ID        = 0x1A,
	// setpoints
	SETROLLSP_SUBTYPE_ID      = 0x1B,
	SETPITCHSP_SUBTYPE_ID     = 0x1C,
	SETYAWSP_SUBTYPE_ID       = 0x1D,
	SETALTSP_SUBTYPE_ID       = 0x1E,
	SETLONSP_SUBTYPE_ID       = 0x1F,
	SETLATSP_SUBTYPE_ID       = 0x20,
	// GETTERS
	// angle
	GETROLLP_SUBTYPE_ID       = 0x00,
	GETROLLI_SUBTYPE_ID       = 0x01,
	GETROLLD_SUBTYPE_ID       = 0x02,
	GETPITCHP_SUBTYPE_ID      = 0x03,
	GETPITCHI_SUBTYPE_ID      = 0x04,
	GETPITCHD_SUBTYPE_ID      = 0x05,
	GETYAWP_SUBTYPE_ID        = 0x06,
	GETYAWI_SUBTYPE_ID        = 0x07,
	GETYAWD_SUBTYPE_ID        = 0x08,
	// rate
	GETROLLRATEP_SUBTYPE_ID   = 0x09,
	GETROLLRATEI_SUBTYPE_ID   = 0x0A,
	GETROLLRATED_SUBTYPE_ID   = 0x0B,
	GETPITCHRATEP_SUBTYPE_ID  = 0x0C,
	GETPITCHRATEI_SUBTYPE_ID  = 0x0D,
	GETPITCHRATED_SUBTYPE_ID  = 0x0E,
	GETYAWRATEP_SUBTYPE_ID    = 0x0F,
	GETYAWRATEI_SUBTYPE_ID    = 0x10,
	GETYAWRATED_SUBTYPE_ID    = 0x11,
	// local pos
	GETLOCALXP_SUBTYPE_ID     = 0x12,
	GETLOCALXI_SUBTYPE_ID     = 0x13,
	GETLOCALXD_SUBTYPE_ID     = 0x14,
	GETLOCALYP_SUBTYPE_ID     = 0x15,
	GETLOCALYI_SUBTYPE_ID     = 0x16,
	GETLOCALYD_SUBTYPE_ID     = 0x17,
	GETALTP_SUBTYPE_ID        = 0x18,
	GETALTI_SUBTYPE_ID        = 0x19,
	GETALTD_SUBTYPE_ID        = 0x1A,
	// setpoints
	GETROLLSP_SUBTYPE_ID      = 0x1B,
	GETPITCHSP_SUBTYPE_ID     = 0x1C,
	GETYAWSP_SUBTYPE_ID       = 0x1D,
	GETALTSP_SUBTYPE_ID       = 0x1E,
	GETLONSP_SUBTYPE_ID       = 0x1F,
	GETLATSP_SUBTYPE_ID       = 0x20,
	// RESPONSE
	// angle
	RESPROLLP_SUBTYPE_ID       = 0x00,
	RESPROLLI_SUBTYPE_ID       = 0x01,
	RESPROLLD_SUBTYPE_ID       = 0x02,
	RESPPITCHP_SUBTYPE_ID      = 0x03,
	RESPPITCHI_SUBTYPE_ID      = 0x04,
	RESPPITCHD_SUBTYPE_ID      = 0x05,
	RESPYAWP_SUBTYPE_ID        = 0x06,
	RESPYAWI_SUBTYPE_ID        = 0x07,
	RESPYAWD_SUBTYPE_ID        = 0x08,
	// rate
	RESPROLLRATEP_SUBTYPE_ID   = 0x09,
	RESPROLLRATEI_SUBTYPE_ID   = 0x0A,
	RESPROLLRATED_SUBTYPE_ID   = 0x0B,
	RESPPITCHRATEP_SUBTYPE_ID  = 0x0C,
	RESPPITCHRATEI_SUBTYPE_ID  = 0x0D,
	RESPPITCHRATED_SUBTYPE_ID  = 0x0E,
	RESPYAWRATEP_SUBTYPE_ID    = 0x0F,
	RESPYAWRATEI_SUBTYPE_ID    = 0x10,
	RESPYAWRATED_SUBTYPE_ID    = 0x11,
	// local pos
	RESPLOCALXP_SUBTYPE_ID     = 0x12,
	RESPLOCALXI_SUBTYPE_ID     = 0x13,
	RESPLOCALXD_SUBTYPE_ID     = 0x14,
	RESPLOCALYP_SUBTYPE_ID     = 0x15,
	RESPLOCALYI_SUBTYPE_ID     = 0x16,
	RESPLOCALYD_SUBTYPE_ID     = 0x17,
	RESPALTP_SUBTYPE_ID        = 0x18,
	RESPALTI_SUBTYPE_ID        = 0x19,
	RESPALTD_SUBTYPE_ID        = 0x1A,
	// setpoints
	RESPROLLSP_SUBTYPE_ID      = 0x1B,
	RESPPITCHSP_SUBTYPE_ID     = 0x1C,
	RESPYAWSP_SUBTYPE_ID       = 0x1D,
	RESPALTSP_SUBTYPE_ID       = 0x1E,
	RESPLONSP_SUBTYPE_ID       = 0x1F,
	RESPLATSP_SUBTYPE_ID       = 0x20,
	// UPDATE
	UPDATE_SUBTYPE_ID          = 0x00,
	BEGINUPDATE_SUBTYPE_ID     = 0x01,
	// LOG
	LOG_SUBTYPE_ID             = 0x00,
	RESPONSE_SUBTYPE_ID        = 0x01
};

// MESSAGE SUBTYPES
struct MessageSubtype{
	char ID;
	char cmdText[100];
	char cmdDataType;
	command_cb * functionPtr;
};

// MESSAGE TYPES
struct MessageType{
	char ID;
	struct MessageSubtype subtypes[MAX_SUBTYPE];
};

/* Defined in commands.c */
extern struct MessageType MessageTypes[MAX_TYPE];

/* Legacy functions - no idea what uses these. Please do not delete. */
float getFloat(unsigned char* str, int pos);
int getInt(unsigned char* str, int pos);
/* end legacy crap */

#endif
