#include <stdio.h>
#include <unistd.h>
#include <getopt.h>

#include "cli_setsetpoint.h"

int cli_set(struct backend_conn * conn,	int argc, char **argv) {
	int c;
	static int setX = 0, setY = 0, setZ = 0;
	struct frontend_setpoint_data setpoint_data;
	static int needHelp = 0;
	static int mask;
	static struct option long_options[] = {
 		/* These options don’t set a flag. We distinguish them by their indices. */
 		{"help",	no_argument,	&needHelp,	1},
 		{0, 0, 0, 0}
 	};

 	while (1)
	{
		/* getopt_long stores the option index here. */
		int option_index = 0;

		c = getopt_long(argc, argv, "x:y:z:", long_options, &option_index);

		/* Detect the end of the options. */
		if (c == -1)
			break;

		switch(c) {
			case 'x' :
				setpoint_data.x  = atof(optarg);
				mask |= SET_X;
				break;
			case 'y' :
				setpoint_data.y  = atof(optarg);
				mask |= SET_Y;
				break;
			case 'z' :
				setpoint_data.z  = atof(optarg);
				mask |= SET_Z;
				break;
			default :
				break;
		}
	}

	if (needHelp) {
		printf("Setsetpoint sets the x, y , or z set point values for the quad.\n");
		printf("Usage Syntax : \n\t./Cli setsetpoint [options...]\n");
		printf("Symlink Usage Syntax : \n\t./setsetpoint [options...]\n\n");
		printf("Available options include the following\n");
		printf("\t[-x] 'val' : Sets the x setpoint value to 'val'\n");
		printf("\t[-z] 'val' : Sets the y setpoint value to 'val'\n");
		printf("\t[-z] 'val' : Sets the z setpoint value to 'val'\n");
		return 0;
	}

	if (argc < 2) {
		printf("Incorrect Usage, run './cli setpid --help' for correct usage.\n");
		return 1;
	}
	
	frontend_setsetpoint(conn, &setpoint_data, mask);
	return 0;
}
